<?php
/**
 * @author    JoomlaShine.com http://www.joomlashine.com
 * @copyright Copyright (C) 2008 - 2012 JoomlaShine.com. All rights reserved.
 * @license   GNU/GPL v2 http://www.gnu.org/licenses/gpl-2.0.html
 */

defined('_JEXEC') or die('Restricted access');

include_once(dirname(__FILE__).DS.'jsn_updater_helper.php');
include_once(dirname(__FILE__).DS.'jsn_httpsocket.php');
include_once(dirname(__FILE__).DS.'jsn_downloadtemplatepackage.php');

class JSNProUpgradeHelper extends JSNUpdaterHelper
{
	var $_template_version      = '';
	var $_edition               = '';
	var $_identified_name       = '';
	var $_joomla_version        = '';
	var $_full_name             = '';
	var $_name                  = '';

	function JSNProUpgradeHelper($templateInfo, $joomlaVersion)
	{
		parent::JSNUpdaterHelper();

		$this->_template_version      = $templateInfo['version'];
		$this->_identified_name       = str_replace('jsn', 'tpl', strtolower($templateInfo['name']));
		$this->_joomla_version        = $joomlaVersion;
		$this->_full_name             = strtolower($templateInfo['full_name']);
		$this->_name                  = strtolower($templateInfo['name']);
	}

	function getPackageFromUpload()
	{
		$install_file = JRequest::getVar('package', null, 'files', 'array');

		if (!(bool) ini_get('file_uploads'))
		{
			$msg 	= 'File upload function is disabled, please enable it in file "php.ini"';
			JError::raiseWarning('SOME_ERROR_CODE', JText::_($msg));
			return false;
		}
		if (!extension_loaded('zlib'))
		{
			$msg = 'Zlib library is disabled, please enable it in file "php.ini"';
			JError::raiseWarning('SOME_ERROR_CODE', JText::_($msg));
			return false;
		}
		if ($install_file['name'] == '')
		{
			$msg 	= 'Installation package is not selected, please download and select it';
			JError::raiseWarning('SOME_ERROR_CODE', JText::_($msg));
			return false;
		}
		if (JFile::getExt($install_file['name']) != 'zip')
		{
			$msg = 'The package has incorrect format, please use exactly the file you downloaded';
			JError::raiseWarning('SOME_ERROR_CODE', JText::_($msg));
			return false;
		}

		$tmp_dest 	= JPATH_ROOT.DS.'tmp'.DS.$install_file['name'];
		$tmp_src	= $install_file['tmp_name'];

		if (!JFile::upload($tmp_src, $tmp_dest))
		{
			$msg = 'Folder "tmp" is Unwritable, please set it to Writable (chmod 777). You can set the folder back to Unwritable after sample data installation';
			JError::raiseWarning('SOME_ERROR_CODE', JText::_($msg));
			return false;
		}

		return $install_file['name'];
	}

	/**
	 * This function will authenticate the user as JoomlaShine customer and
	 * return the array of versions of template that the user purchased
	 * @return [type] [description]
	 */
	function authenticateCustomerInfo()
	{
		$post						= JRequest::get('post');
		$post['customer_password'] 	= JRequest::getString('customer_password', '', 'post', JREQUEST_ALLOWRAW);
		$link						= JSN_TEMPLATE_AUTOUPDATE_URL.'&identified_name='.urlencode($this->_identified_name).'&joomla_version='.urlencode($this->_joomla_version).'&username='.urlencode($post['customer_username']).'&password='.urlencode($post['customer_password']).'&upgrade=no';
		$objHTTPSocket 				= new JSNHTTPSocket($link, null, null, 'get');
		$result    					= $objHTTPSocket->socketDownload();

		$errorCode = strtolower((string) $result);
		$hasError = true;
		
		$rel = new stdClass();
		if ($result)
		{
			switch ($errorCode)
			{
				case 'err00':
					$message = 'Invalid Parameters! Cannot verify your product information.';
					break;
				case 'err01':
					$message = 'Invalid username or password. Please enter JoomlaShine customer account you created when you purchased the product.';
					break;
				case 'err02':
					$message = 'Installation is not authorized. We could not find the product in your order list. Seems like you did not purchase it yet.';
					break;
				case 'err03':
					$message = 'Requested file could not be found on server.';
					break;
				default:
					$hasError = false;
					break;
			}

			if ($hasError === true)
			{
				$rel->error = true;
				$rel->message = $message;
			}
			else
			{
				/* Standardize the returned array */
				$result = json_decode($result, true);
				$editionArray = array();
				foreach ($result['editions'] as $value) {
					if (!in_array($value, $editionArray))
					{
						$editionArray[] = $value;
					}
				}

				$rel->error                     = false;
				$rel->editions                  = $editionArray;
				$rel->post['customer_username'] = $post['customer_username'];
				$rel->post['customer_password'] = $post['customer_password'];
			}
		}
		else
		{		
			$rel->error = true;
			$rel->message = 'Can not authorize your Customer account! Your server does not allow connection to Joomlashine server.';
		}

		return $rel;
	}

	function downloadTemplatePackage($post, $edition)
	{
		$link					= JSN_TEMPLATE_AUTOUPDATE_URL.'&identified_name='.urlencode($this->_identified_name).'&edition='.urlencode($edition).'&joomla_version='.urlencode($this->_joomla_version).'&username='.urlencode($post->customer_username).'&password='.urlencode($post->customer_password).'&upgrade=yes';
		$tmpName				= strtolower($this->_name.'_'.str_replace(' ', '_', $edition).'.zip');
		$objJSNDownloadPackage  = new JSNDownloadTemplatePackage($link, $tmpName);
		
		return $objJSNDownloadPackage->download();
	}

	function migrateSettings($freeParams)
	{
		/* Migrate params settings from FREE to PRO edition */
		$proParams = $this->_obj_utils->getTemplateParameters($this->_name . '_pro');
		
		$freeParams = parse_ini_string($freeParams->_raw);
		$proParams  = parse_ini_string($proParams->_raw);
		
		if ($freeParams && $proParams)
		{
			foreach ($freeParams as $key => $value)
			{
				if (isset($proParams[$key]) && $proParams[$key] != $value) 
				{
					$proParams[$key] = $value;
				}
			}

			$proParamFile = JPATH_ROOT.DS.'templates'.DS.$this->_name.'_pro'.DS.'params.ini';

			// Set FTP credentials, if given
			jimport('joomla.client.helper');
			JClientHelper::setCredentialsFromRequest('ftp');
			$ftp = JClientHelper::getCredentials('ftp');

			jimport('joomla.filesystem.file');
			if (JFile::exists($proParamFile))
			{
				$registry = new JRegistry();
				$registry->loadArray($proParams);
				$txt = $registry->toString();

				// Try to make the params file writeable
				if (!$ftp['enabled'] && JPath::isOwner($proParamFile) && !JPath::setPermissions($proParamFile, '0755')) {
					return JText::_('Could not make the template parameter file writable');
				}

				if (!JFile::write($proParamFile, $txt))
				{
					return 'Could not write params.ini file!';
				}

				// Try to make the params file unwriteable
				if (!$ftp['enabled'] && JPath::isOwner($proParamFile) && !JPath::setPermissions($proParamFile, '0644')) {
					return JText::_('Could not make the template parameter file unwritable');
				}
			}
		}

		/* Enable the new PRO template as default */
		$db =& JFactory::getDBO();
		$query  = 'DELETE FROM #__templates_menu'
					. ' WHERE template != "' . $this->_name . '_pro"'
					. ' AND client_id = 0';
		$db->setQuery($query);
		$db->query();
		if ($db->getErrorMsg()) {
			return $db->getErrorMsg();
		}

		$query = 'REPLACE INTO #__templates_menu'
					. '(template, menuid, client_id)'
					. ' VALUES (' . $db->quote($this->_name.'_pro') . ', 0, 0)';
		$db->setQuery($query);
		$db->query();
		if ($db->getErrorMsg()) {
			return $db->getErrorMsg();
		}	

		return true;
	}

	function destroyUpgradeSession($sessionNameArray, $noJoomlaLogin = false)
	{
		if (is_array($sessionNameArray))
		{
			$session = &JFactory::getSession();
			if ($noJoomlaLogin === true)
			{
				foreach ($sessionNameArray as $sessionKey => $sessionName) {
					if ($sessionKey != 'joomla_login')
					{
						$session->clear($sessionName, 'jsntemplatesession');
					}
				}
			}
			else
			{
				foreach ($sessionNameArray as $sessionName) {
					$session->clear($sessionName, 'jsntemplatesession');
				}
			}
		}
	}
}